package Foswiki::Plugins::HijaxPlugin;

# Always use strict to enforce variable scoping
use strict;
use JSON;

require Foswiki::Func;    # The plugins API
require Foswiki::Plugins; # For the API version
require Foswiki::Attrs;

use vars qw( $VERSION $RELEASE $SHORTDESCRIPTION $debug $pluginName $NO_PREFS_IN_TOPIC );

$VERSION = '$Rev: 15942 (11 Aug 2008) $';
$RELEASE = '1.0';
$SHORTDESCRIPTION = 'ajax integration';
$NO_PREFS_IN_TOPIC = 1;

# Name of this Plugin, only used in this module
$pluginName = 'HijaxPlugin';


sub initPlugin {
    my( $topic, $web, $user, $installWeb ) = @_;

    # check for Plugins.pm versions
    if( $Foswiki::Plugins::VERSION < 1.026 ) {
        Foswiki::Func::writeWarning( "Version mismatch between $pluginName and Plugins.pm" );
        return 0;
    }

    $debug = $Foswiki::cfg{Plugins}{HijaxPlugin}{Debug} || 0;

    return 1;
}

sub _noTemplateMessage {
	my $tmpl = shift;
	return "<div class='FoswikiAlert'>Warning: a template named $tmpl has not been defined for this interface.</div>";
}


sub completePageHandler {
    #my($html, $httpHeaders) = @_;
    # modify $_[0] or $_[1] if you must change the HTML or headers

    # Foswiki::Func::writeDebug( "- ${pluginName}::completePageHandler \n$_[0]" ) if $debug;
	my $query = Foswiki::Func::getCgiQuery();
	return unless $query->param('cover') && $query->param('cover') =~ /ajax/ && $_[0] =~ /%TEMPLATE:/;
		
	my @hijax = ();
	my @tmplp = ();
	if (Foswiki::Func::getContext()->{'login'}) {
		@tmplp = ('content','errorstep');
	} else {
		@hijax = $query->param('hijax') if defined $query->param('hijax');
		@tmplp = $query->param('tmplp') if defined $query->param('tmplp');
		$tmplp[0] = 'content' unless ($hijax[0] || $tmplp[0]); # if nothing else, we return the 'content' tmpl
		if (Foswiki::Func::getContext()->{'oops'}) {
			push (@tmplp,'content') unless grep (/content/,@tmplp);
		}
	}

	my $response = {}; # the hash we return as JSON
	my ($remainder, $text) = split(/%TEMPLATE:/,$_[0],2);
	my @templates = split(/%TEMPLATE:/,$text);
	foreach my $record (@templates) {
		my ($templateName, $templateText) = split(/%/,$record,2);
		if (grep(/$templateName/, @tmplp)) {
			$response->{$templateName} = $templateText;
		}
	}
	foreach my $request (@tmplp) {
		$response->{$request} ||= _noTemplateMessage($request);
	}
	foreach my $request (@hijax) {
		my $templatetext = Foswiki::Func::expandTemplate($request);
		if ($templatetext) {
			$templatetext = 
				Foswiki::Func::renderText(Foswiki::Func::expandCommonVariables($templatetext));
		} else {
			$templatetext = _noTemplateMessage($request);
		}
		$response->{$request} = $templatetext;
	}
	$response->{context} = Foswiki::Func::expandTemplate('context');
	
	$_[0] = to_json($response);
}

sub redirectCgiQueryHandler {
    # do not uncomment, use $_[0], $_[1] instead
    ### my ( $query, $url ) = @_;
	if ($debug) {
		my $query_string = $_[0]->query_string();
	    Foswiki::Func::writeDebug( "- ${pluginName}::redirectCgiQueryHandler( $query_string, $_[1] )" );
	}
    return if $_[0]->param('foswiki_redirect_cache'); # our params have already been cached for the other side
	return unless $_[0]->param('cover') =~ /ajax/;
	my $existing = '';
	my $append = '?';
	my $cover = '';
	if ($_[1] =~ s/(\?.*)$//) {
		$existing = $1;
		if ($existing =~ /cover=([\w,\s]+)/) {
			$cover = $1;
		}
		$append = '&';
	}
	if ($cover eq '') {
		$existing .= $append.'cover=ajax';
	} elsif ($cover !~ /ajax/) {
		$_[0]->param(-name => 'cover', -value => 'ajax,'.$cover);
		$existing =~ s/$cover/ajax,$cover/;
	}
	$_[1] .= $existing;
	$_[1] .= '&hijax='.join('&hijax=',$_[0]->param('hijax')) if $_[0]->param('hijax');
	$_[1] .= '&tmplp='.join('&tmplp=',$_[0]->param('tmplp')) if $_[0]->param('tmplp');
	if ($debug) {
		my $query_string = $_[0]->query_string();
	    Foswiki::Func::writeDebug( "- ${pluginName}::redirectCgiQueryHandler( $query_string, $_[1] )" );
	}
	return 0;
}

1;
